/***************************************************************************
                          ppmcdiag.c  -  description
                             -------------------
    begin                : Mon Oct 23 2000
    copyright            : (C) 2000 by Eric Keller
    email                : eek105
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "ppmc.h"
#include "extintf.h"
#include <asm/io.h>

unsigned int ppmc_base_addr=0x378;

#define CONTROLPORT ppmc_base_addr+2
#define DATAPORT ppmc_base_addr+4

/* ident tag */
static char ident[] = "$Id: ppmcdiag.c,v 3.2 2003/03/11 19:29:58 jmelson Exp $";


/*
  Be root to run it
  */

#define NUM_AXES 4
enum {POS = 1, LIMIT, HOME, INDEX, LATCH, DAC, SERVO,SERVOF,DIO,DIOCONT,BUS,COMM};
static int done = 0;
static void quit(int sig)
{
  done = 1;
}




int main(int argc, char *argv[])
{
  int which = POS;
  int newline=0;
  double pos[NUM_AXES];
  double oldpos[NUM_AXES];
  int lim[NUM_AXES*2];
  int oldlim[NUM_AXES*2];
  int home[NUM_AXES];
  int oldhome[NUM_AXES];
  double dacval[NUM_AXES];
  int index;
  int latch;
  int axis,errors;
  double volts;
  int t,var;
  unsigned int a,b;
  char buf[80];
  char *cptr;
  //printf("diags ppmc test program V3.2\n");
  if (argc==1)
	{
      printf("syntax: %s {pos | dac | limit | home | index <axis> | latch <axis> | servo{f} | DIO | diocontinuous | BUS | commtest}\n", argv[0]);
	    exit(-1);   //the program was hard to write, it should be hard to use
	}
	if (argc > 1)
  {
      if (!strcmp(argv[1], "pos"))
	    {
	  		which = POS;
			}
      else if (!strcmp(argv[1], "dac"))
			{
	  		which = DAC;
			}
      else if (!strcmp(argv[1], "limit"))
			{
	  		which = LIMIT;
			}
      else if (!strcmp(argv[1], "home"))
			{
	  		which = HOME;
			}
      else if (!strcmp(argv[1], "servo"))
			{
	  		which = SERVO;
			}
      else if (!strcmp(argv[1], "servof"))
			{
	  		which = SERVOF;
			}
      else if (!strcmp(argv[1], "bus"))
			{
	  		which = BUS;
			}
      else if (!strcmp(argv[1], "diocontinuous"))
			{
	  		which = DIOCONT;
			}
      else if (!strcmp(argv[1], "commtest"))
			{
	  		which = COMM;
			}
      else if (!strcmp(argv[1], "index"))
			{
	  		if (1 == sscanf(argv[2], "%d", &axis))
	    	{
	      	which = INDEX;
	    	}
	  		else
	    	{
	      	printf("syntax: %s {pos | dac | limit | home | index <axis> | latch <axis>}"
		       "| servo{f} | dio | bus\n", argv[0]);
	      	exit(-1);
	    	}
			}
      else if (!strcmp(argv[1], "latch"))
			{
	  		if (1 == sscanf(argv[2], "%d", &axis))
	    	{
	      	which = LATCH;
	    	}
	  		else
	    	{
	      	printf("syntax: %s {pos | dac | limit | home | index <axis> | latch <axis>}\n", argv[0]);
	      	exit(-1);
	    	}
		}
      else if (!strcmp(argv[1], "dio"))
	{
	  which = DIO;
	}
     else
		{
	      printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(-1);
		}
  } //end if argc>1


  signal(SIGINT, quit);

  ppmcMotInit(NULL);
  printf("parport addr 0x%x \n", ppmc_base_addr);

  if (which == POS)
  {
      while (! done)
			{
	  			newline = 0;
	  			ppmcEncoderReadAll(NUM_AXES, pos);
	  			for (t = 0; t < NUM_AXES; t++)
	    		{
	      		if(fabs(pos[t]-oldpos[t]) > 1000.0 && !newline)
						{
		  				newline=1;
		  				printf("\n");
						}
	      		oldpos[t] = pos[t];
	    		}
	  			for (t = 0; t < NUM_AXES; t++)
	    		{
	      		printf("%+15.3f\t", pos[t]);
	    		}
	  			printf("\r");
	  			fflush(stdout);
	  			usleep(10000);
			}
  }
  if (which == SERVO)
  {
      while (! done)
       {
	 ppmcDioWrite(2,1);  // get out of estop
	  newline = 0;
	  ppmcEncoderReadAll(NUM_AXES, pos);
	  for (t = 0; t < NUM_AXES; t++)
          {
          if(fabs(pos[t]-oldpos[t]) > 1000.0 && !newline)
	    {
	    newline=1;
	    printf("\n");
	    }
	  oldpos[t] = pos[t];
       }
       for (t = 0; t < NUM_AXES; t++)
         {
	   printf("%+12.1f ", pos[t]);
           dacval[t] = (pos[t] / 1000.0);
	 }
       printf("   ");
       for (t = 0; t < NUM_AXES; t++)
	 {
	   printf("%+10.2f ",dacval[t]);
	 }
       printf("\n");
       ppmcDacWriteAll(NUM_AXES,dacval);
       fflush(stdout);
       usleep(10000);
    }
  }
  if (which == SERVOF)
  {
      while (! done)
       {
	 ppmcDioWrite(2,1);  // get out of estop
	  newline = 0;
	  ppmcEncoderReadAll(NUM_AXES, pos);
          for (t = 0; t < NUM_AXES; t++)
         {
             dacval[t] = (pos[t] / 1000.0);
	 }
       ppmcDacWriteAll(NUM_AXES,dacval);
    }
  }
  else if (which == LIMIT)
  {
      while (! done)
			{
	  		newline=0;
	  		for (t = 0; t < NUM_AXES; t++)
	    	{
	      	ppmcMaxLimitSwitchRead(1, &lim[2*t]);
	      	if(oldlim[2*t] != lim[2*t] && !newline)
					{
		  			printf("\n");
		  			newline=1;
					}
	      	oldlim[2*t] = lim[2*t];
	      	ppmcMinLimitSwitchRead(1, &lim[2*t+1]);
	      	if(oldlim[2*t+1] != lim[2*t+1] && !newline)
					{
		  			printf("\n");
		  			newline=1;
					}
	      	oldlim[2*t+1] = lim[2*t+1];
	    	}
	  		if(newline)
	    	{
	      	for (t = 0; t < NUM_AXES; t++)
					{
		  			printf("%+d\t", lim[2*t]);
		  			printf("%+d\t", lim[2*t+1]);
					}
	      	printf("\n");
	      	fflush(stdout);
	    }
	  	usleep(10000);
		}
  }
  else if (which == HOME)
  {
      while (! done)
			{
	  		newline=0;
	  		for (t = 0; t < NUM_AXES; t++)
	    	{
	      	ppmcHomeSwitchRead(t, &home[t]);
	      	if(oldhome[t] != home[t] && !newline)
					{
		  			printf("\n");
		 				newline=1;
					}
	      	oldhome[t] = home[t];
	    	}
	  		if(newline)
	    	{
	      		for (t = 0; t < NUM_AXES; t++)
						{
		  				printf("%d\t", home[t]);
						}
	      		printf("\r");
	      		fflush(stdout);
	    	}
	  		usleep(10000);
		}
  }
  else if (which == INDEX)
  {
      if (axis < 0 || axis >= NUM_AXES)
			{
	  		printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(1);
			}

      ppmcEncoderResetIndex(axis);
      while (! done)
			{
	  		ppmcEncoderReadLevel(axis, &index);
	  		printf("%d ", index);
	  		fflush(stdout);
	  		usleep(10000);
			}
  }
  else if (which == LATCH)
  {
      if (axis < 0 || axis >= NUM_AXES)
			{
	  		printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(1);
			}

      ppmcEncoderResetIndex(axis);
      while (! done)
			{
				ppmcEncoderReadLatch(axis, &latch);
				printf("%d ", latch);
				fflush(stdout);
				usleep(10000);
			}
	}
  else if (which == DAC)
	{
      printf("Axis? ");
      fgets(buf,80,stdin);
      axis = (int) strtol(buf,0,0);
      while( !done)
			{
				printf("DAC(%d)",axis);
				printf("volts? ");
				if(fgets(buf,80,stdin) == NULL)
				{
					printf("got null in dac\n");
					break;
				}
				if(!isdigit(buf[0]) && buf[0] != '-')
				{
					printf("non numeric volts , quitting\n");
					break;
				}
				cptr = buf;
				while((*cptr >= '0' && *cptr <= '9')
								|| *cptr == '-' || *cptr == '.' || *cptr == 'E')
				{
					cptr++;
				}
	  		*cptr = 0;
	  		volts = strtod(buf,0);
	  		printf("volts=%f\n",volts);
	  		ppmcDacWrite(axis, volts);
			}
  }  //end if dac
  else if (which == DIO)
    {
      printf("dio \n");
      ppmcDioRead(0, &axis);
      printf("Digital In 0 = ,%d ",axis);
      ppmcDioRead(1, &axis);
      printf("Digital In 1 = ,%d ",axis);
      ppmcDioRead(2, &axis);
      printf("Estop In = ,%d\n",axis);

      printf("Enter 1 to do output");
      fgets(buf,80,stdin);
      axis = (int) strtol(buf,0,0);
      if (axis == 1)
	{
	  printf("value to output ?");
	  if(isdigit(buf[0]))
	    {
	      ppmcDioWrite(2,1);     // turn off E-stop
	      cptr = buf + 1;
	      axis = (int) strtol(cptr,0,0);
	      printf("sending %d\n",axis);
	      ppmcDioWrite(0,axis);
	    }
	}
    }
  else if (which == DIOCONT)
    {
      var = 0;     // bloop variable
      t = 0;     //slow down counter
      ppmcDioWrite(2,1);  // get out of estop
      printf("  DIO 0 inputs        DIO 1 inputs\n");
      printf(" 7 6 5 4 3 2 1 0     7 6 5 4 3 2 1 0\n");
      while (! done)
	{
	  ppmcDioRead(0, &axis);
	  for (a = 8 ; a--; a >= 0) {
     	    printf("%2d",(axis>>a)&0x01);
	  }
	  printf("    ");
	  ppmcDioRead(1, &axis);
	  for (a=8; a--; a >= 0) {
	    printf("%2d",(axis>>a)&0x01);
	  }
	  t++;
	  if (t > 500 ) {
	    t=0;
	    var = var<<1;
	    if ((var&0x100) != 0) {
	      var = var & 0xFE;
	    } else {
	      var = var | 1; }
	    ppmcDioWrite(0,var);    // write to output port
	    printf("  output %x",var);
	    
	  }
	  printf("\r");

	}
    }
   else if (which == BUS)
     {
       printf("Bus Map \n  Board Addr  Type         Ver.\n");
       for (a = 15; a<=255; a+=16)
	 {
	   SelAddr(a);
	   outb(0x24,CONTROLPORT);  // set EPP port to input mode
	   b = inb(DATAPORT);
	   switch (b>>4)
	     {
	       case 0 : printf("%4x        Unknown       %4x\n",(a>>4),b); break;
	       case 1 : printf("%4x        Encoder       %4x\n",(a>>4),(b & 15)); break;
	       case 2 : printf("%4x        DAC-16        %4x\n",(a>>4),(b & 15)); break;
	       case 3 : printf("%4x        DIO           %4x\n",(a>>4),(b & 15)); break;
	       case 4 : {
		 printf("%4x        Univ. Stepper %4x\n",(a>>4),(b & 15));
		 a+=16;      // skip next addr as univ stepper uses 2 blocks
		 break;
	       }
	       case 15 : printf("%4x        No Board      %4x\n",(a>>4),b); break;
	       default : printf("%4x        Unknown       %4x\n",(a>>4),b); break;

	     }
	 }
     }
  else if (which == COMM)
    {
      t =0;
      errors=0;
      while (! done) {
	t++;
	if ((t%1000) == 0)
	  printf(" %d test cycles, %d errors\n",t,errors);
	// aughhh !! big goof in decode of latstb in univstep!
	SelWrt(ENCCTRL_0,0xF0);  // select load pos counter and enable
	SelWrt(ENCCTRL_0,0xF0);  // and hold state for a while
	home[0] = rand() & 0xff;
	home[1] = rand() & 0xff;
	home[2] = rand() & 0xff;
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCLOAD_0,home[0]);     // load data into holding register
	WriteData(home[1]);
	WriteData(home[2]);
	SelWrt(ENCCTRL_0,0);  // and turn the load pos counter off
	//	ppmcEncoderReadAll(NUM_AXES, pos);
	EncoderLatch();  // latch data into storage registers
	SelAddr(ENCCNT0);   // some delay before real read
	SelAddr(ENCCNT0);
	SelAddr(ENCCNT0);
	outb(0x24,CONTROLPORT);    // set EPP to input mode
	for (a = 0; a < NUM_AXES; a++)
	  {
	    oldhome[0]=ReadData();
	    oldhome[1]=ReadData();
	    oldhome[2]=ReadData();
	    if ((home[0] != oldhome[0]) || (home[1] != oldhome[1]) || (home[2] != oldhome[2])) 
	      {
	      printf(" test cycle %d, axis %d was / should be \n",t,a);
	      printf(" %2x %2x %2x / %2x %2x %2x \n",oldhome[0],oldhome[1],oldhome[2],home[0],home[1],home[2]);
	      }
	  }	
      }
    }
  //ppmcAioQuit();
  //ppmcDioQuit();
  extMotQuit();

  return 0;
}


